/*
	WiTilt v4 Firmware  
	
    Written by Ryan Owens    

    If you read over this material and pickup even a smidge of new knowledge, please 
    reciprocate the favor and include Spark Fun Electronics in the  publication, 
    documentation, website, or whatever other communication conduit you choose. If 
    you use any of our firmware, schematics (whole or in part), methods, concepts, 
    or even our thoughts on life, you must include a note about Spark Fun Electronics 
    so that a few other people can hear about our funky little company. 
     
*/

#include <stdlib.h>

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <math.h>
#include "main.h"

//Global Variables
//===============================================================
ConfigStruct Config;	//Contains Configuration Settings
SensorStruct TriAx;		//Contains Information Pertaining to the TriAx Sensor3
GyroStruct Gyro;		//Contains Information Pertaining to the Gyroscope
CalStruct Calibration;	//Contains TriAx Sensor Calibration Information

volatile uint8_t values_updated=0;
volatile uint8_t calibration_flag=0;

//Define Functions
//================================================================
void ioinit(void);      // initializes IO
static int uart_putchar(char c, FILE *stream);
uint8_t uart_getchar(void);
static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

void print_decimal(uint32_t number, uint8_t place, uint8_t pad);
static int witilt_putchar(char c);
void delay_ms(uint16_t x); // general purpose delay

uint16_t get_TriAx_Val(uint8_t axis);

void config_menu(void);

void rawDisplay(void);
void binaryDisplay(void);
void degreeDisplay(void);

void calibrate(void);

void change_range_mode(void);
void set_range(void);

void set_threshold(void);

void change_display_mode(void);

void set_output_freq(void);

void activate_channels(void);
void config_bluetooth(void);
void write_to_EEPROM(unsigned int Address, unsigned char Data);
unsigned char read_from_EEPROM(unsigned int Address);
void print_decimal(uint32_t number, uint8_t place, uint8_t pad);

//SPI Functions
uint8_t spi_exch(uint8_t output);
uint16_t spi_full(uint8_t cmd, uint16_t sdat);
uint16_t adcc(uint8_t value);
uint16_t adcr(void);
uint16_t adc_Read(uint8_t value);


//================================================================

//Display Strings
//================================================================
const char wlcm_str[] PROGMEM = "\n\n\rWiTilt v4.0 - Configuration Menu:\n\r=========================\n\r [1] Start Tri-Ax detection (Press any key to stop)\n\r [2] Set Active Channels (";
const char menu_options1[] PROGMEM = " [3] Calibrate\n\r [4] Sensor Range ("; 
const char range_mode_str[] PROGMEM = "Change Range Mode to:\n\r[1]1_5g Range\n\r[2]6g Range Mode\n\r[x]Exit\n\r";
const char change_disp_str[] PROGMEM = "Change Display Mode to:\n\r[1]Gravity Values\n\r[2]Raw Values\n\r[3]Binary Mode\n\r[4]Degree Mode\n\r";
const char rotatex_str[] PROGMEM = "\n\rRotate on the X-Axis.\n\rPress a button at the Max.\n\r";
const char rotatey_str[] PROGMEM = "\n\rRotate on the Y-Axis.\n\rPress a button at the Max.\n\r";
const char rotatez_str[] PROGMEM = "\n\rRotate on the Z-Axis.\n\rPress a button at the Max.\n\r";
const char rotater_str[] PROGMEM = "\n\rHold device for Zero rotational movement.\n\r";
const char calibrate_min_str[] PROGMEM = "\n\rPress a button at the Min.\n\r";
const char activateZ[] PROGMEM = "Z-Tilt Active?(y/n)\n\r";
const char activateB[] PROGMEM = "Battery Voltage Active?(y/n)\n\r";
const char activateY[] PROGMEM = "Y-Tilt Active?(y/n)\n\r";
const char activateX[] PROGMEM = "X-Tilt Active?(y/n)\n\r";
const char activateR[] PROGMEM = "Gyroscope Active?(y/n)\n\r";
const char incrdecr[] PROGMEM = "(i)ncrease\n\r(d)ecrease\n\rPress 'x' to select.\n\r";
//================================================================

ISR (TIMER1_COMPA_vect)
{
	cli();												//Disable Interrupts
	TCCR1B=(1<<WGM12)|(0<<CS11)|(0<<CS10)|(0<<CS12);	//Stop the Timer
	if(TriAx.x_status==1)TriAx.x_val=get_TriAx_Val('x');
	if(TriAx.y_status==1)TriAx.y_val=get_TriAx_Val('y');
	if(TriAx.z_status==1)TriAx.z_val=get_TriAx_Val('z');
	if(TriAx.b_status==1)TriAx.b_val=get_TriAx_Val('b');
	if(Gyro.status==1)adc_Read(RATE_CHANNEL);
	values_updated=1;									//tell the display function that new values are available.
	TCCR1B=(1<<WGM12)|(1<<CS11)|(1<<CS10);			//Start the timer
	sei();												//Enable Interrupts
}

int main (void)
{	
    ioinit(); //Boot up defaults and configure Bluetooth connection
	delay_ms(500);
	config_bluetooth();

	while(1)
    {
		sbi(PORTD, STATUS_LED);
		config_menu();
		delay_ms(100);
		cbi(PORTD, STATUS_LED);
		delay_ms(100);
	}
    return (0);
}


void config_menu(void)
{
    uint8_t choice=0;
    
    while(1)
    {
		printf_P(wlcm_str);
		
		if(TriAx.x_status == ACTIVE) printf("X");
        else printf("-");
        if(TriAx.y_status == ACTIVE) printf("Y");
        else printf("-");
        if(TriAx.z_status == ACTIVE) printf("Z");
        else printf("-");
		if(TriAx.b_status == ACTIVE) printf("B");
		else printf("-");
		if(Gyro.status == ACTIVE) printf("R");
		else printf("-");
        printf(" Active)\r\n");
        
        printf_P(menu_options1);
		
        if(Config.range_mode == RANGE_15G) printf("1.5g");
        if(Config.range_mode == RANGE_6G) printf("6g");
        printf(")\n\r");
        
        printf(" [5] Display Mode (");        
        if(Config.display_mode == GRAVITY) printf("Gravity");
        if(Config.display_mode == RAW) printf("Raw");
        if(Config.display_mode == BINARY) printf("Binary");
		if(Config.display_mode == DEGREE) printf("Degree");
        printf(")\n\r");
        
        printf(" [6] Set/View Threshold Values\n\r");
		
        printf(" [7] Set Output Frequency (%ldHz)\n\r", (divided_clock/Config.delay_amount)-(divided_clock/Config.delay_amount)%10);
        printf("\n\r : ");
				
        choice = uart_getchar();
		putchar(choice);
		putchar('\n');
		putchar('\r');
		if(choice=='1')
		{
			sei();
			if((Config.display_mode == RAW)||(Config.display_mode==GRAVITY))rawDisplay();
			if(Config.display_mode==BINARY) binaryDisplay();
			if(Config.display_mode==DEGREE) degreeDisplay();
			cli();
		}
		if(choice=='2')activate_channels();
		if(choice=='3')calibrate();
		if(choice=='4')change_range_mode();
		if(choice=='5')change_display_mode();
		if(choice=='6')set_threshold();
		if(choice=='7')set_output_freq();
    }
}

//Core functions
//==================================================
void ioinit(void)
{
	uint8_t RunCount=0;

    //1 = output, 0 = input
    DDRB = 0b11101111; //PB4 = MISO 
    DDRC = 0b00100000; //PC01234 67 = AX AY AZ VREF TEMP (ST) ERR RAW
    DDRD = 0b11111110; //(PD0 = RX)

    //SPI Bus setup
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPHA)|(1<<CPOL);//|(1<<SPR0);
	SPSR = (1<<SPI2X);
	
	// USART Baud rate: 115200
    UBRR0H = (MYUBRR) >> 8;
    UBRR0L = MYUBRR;
    UCSR0B = (1<<RXEN0)|(1<<TXEN0);
    UCSR0C = (3<<UCSZ00);
    UCSR0A = (1<<U2X0);
    
    stdout = &mystdout; //Required for printf init
    
    //Init timer
    ASSR = 0;
    TCCR1B=(1<<WGM12)|(1<<CS11)|(1<<CS10);
	TIMSK1 = (1<<OCIE1A); //Enable Timer Overflow Interrupt for Timer 1
	
    
	//config_bluetooth();
	for(uint8_t x = 0 ; x < 6 ; x++)
    {
        sbi(PORTD, STATUS_LED);
        delay_ms(100);
        cbi(PORTD, STATUS_LED);
        delay_ms(100);
    }
	
	RunCount=read_from_EEPROM(RunCountAdd);
	if(RunCount==1){	//Check to see if unit has been run before		
		//Read Settings from EEPROM
		//Get Active Accelerometer Axis Values
		TriAx.x_status = read_from_EEPROM(TriAxStartAdd+x_status_addr);
		TriAx.y_status = read_from_EEPROM(TriAxStartAdd+y_status_addr);
		TriAx.z_status = read_from_EEPROM(TriAxStartAdd+z_status_addr);
		TriAx.b_status = read_from_EEPROM(TriAxStartAdd+b_status_addr);
		Gyro.status=read_from_EEPROM(GyroStartAdd+status_addr);
		
		//Get Configuration Settings
		Config.range_mode = read_from_EEPROM(ConfigStartAdd+range_mode_addr);
		Config.delay_amount = read_from_EEPROM(ConfigStartAdd+delay_amount_addr+LOW_BYTE);
		Config.delay_amount |= (read_from_EEPROM(ConfigStartAdd+delay_amount_addr+HIGH_BYTE)<<8);
		Config.display_mode = read_from_EEPROM(ConfigStartAdd+display_mode_addr);
		Config.gthreshold = read_from_EEPROM(ConfigStartAdd+gthreshold_addr);
		Config.threshold = read_from_EEPROM(ConfigStartAdd+threshold_addr);
		Config.rthreshold = read_from_EEPROM(ConfigStartAdd+rthreshold_addr);
		Config.bthreshold = read_from_EEPROM(ConfigStartAdd+bthreshold_addr+LOW_BYTE);
		Config.bthreshold |= (read_from_EEPROM(ConfigStartAdd+bthreshold_addr+HIGH_BYTE)<<8);
		
		//Get Calibration Values
		Calibration.X_Midpoint = read_from_EEPROM(CalStartAdd+x_midpoint_addr+LOW_BYTE);
		Calibration.X_Midpoint |= (read_from_EEPROM(CalStartAdd+x_midpoint_addr+HIGH_BYTE)<<8);
		Calibration.X_1g = read_from_EEPROM(CalStartAdd+x_1g_addr+LOW_BYTE);
		Calibration.X_1g |= (read_from_EEPROM(CalStartAdd+x_1g_addr+HIGH_BYTE)<<8);
	
		Calibration.Y_Midpoint = read_from_EEPROM(CalStartAdd+y_midpoint_addr+LOW_BYTE);
		Calibration.Y_Midpoint |= (read_from_EEPROM(CalStartAdd+y_midpoint_addr+HIGH_BYTE)<<8);
		Calibration.Y_1g = read_from_EEPROM(CalStartAdd+y_1g_addr+LOW_BYTE);
		Calibration.Y_1g |= (read_from_EEPROM(CalStartAdd+y_1g_addr+HIGH_BYTE)<<8);
	
		Calibration.Z_Midpoint = read_from_EEPROM(CalStartAdd+z_midpoint_addr+LOW_BYTE);
		Calibration.Z_Midpoint |= (read_from_EEPROM(CalStartAdd+z_midpoint_addr+HIGH_BYTE)<<8);
		Calibration.Z_1g = read_from_EEPROM(CalStartAdd+z_1g_addr+LOW_BYTE);
		Calibration.Z_1g |= (read_from_EEPROM(CalStartAdd+z_1g_addr+HIGH_BYTE)<<8);
	
		Gyro.zero = read_from_EEPROM(GyroStartAdd+zero_addr+LOW_BYTE);
		Gyro.zero |= (read_from_EEPROM(GyroStartAdd+zero_addr+HIGH_BYTE)<<8);
	}
	else{							//Unit has never been run.  Set initial config. values.
		TriAx.x_status = ACTIVE;
		write_to_EEPROM(TriAxStartAdd+x_status_addr, TriAx.x_status);
		TriAx.y_status = ACTIVE;
		write_to_EEPROM(TriAxStartAdd+y_status_addr, TriAx.y_status);
		TriAx.z_status = ACTIVE;
		write_to_EEPROM(TriAxStartAdd+z_status_addr, TriAx.z_status);
		TriAx.b_status = ACTIVE;
		write_to_EEPROM(TriAxStartAdd+b_status_addr, TriAx.b_status);
		Gyro.status=ACTIVE;
		write_to_EEPROM(GyroStartAdd+status_addr, Gyro.status);
		
		Config.range_mode=RANGE_15G;
		write_to_EEPROM(ConfigStartAdd+range_mode_addr, Config.range_mode);
		Config.delay_amount = divided_clock/135;
		write_to_EEPROM(ConfigStartAdd + delay_amount_addr + LOW_BYTE, (Config.delay_amount & 0xFF));
		write_to_EEPROM(ConfigStartAdd + delay_amount_addr + HIGH_BYTE, ((Config.delay_amount>>8)&0xFF));
		Config.display_mode = GRAVITY;
		write_to_EEPROM(ConfigStartAdd+display_mode_addr, Config.display_mode);
		Config.gthreshold = 0;
		Config.threshold = 0;
		Config.rthreshold = 0;
		Config.bthreshold = 3600;
		write_to_EEPROM(ConfigStartAdd+rthreshold_addr, Config.rthreshold);
		write_to_EEPROM(ConfigStartAdd + gthreshold_addr, Config.gthreshold);
		write_to_EEPROM(ConfigStartAdd + bthreshold_addr + LOW_BYTE, (Config.bthreshold&0XFF));
		write_to_EEPROM(ConfigStartAdd + bthreshold_addr + HIGH_BYTE, ((Config.bthreshold>>8)&0xFF));
		write_to_EEPROM(ConfigStartAdd+threshold_addr, Config.threshold);
		
		write_to_EEPROM(RunCountAdd, 1);	//Set the run count so values don't re-initialize
	}
	
	OCR1AH=(Config.delay_amount>>8);		//Load Compare Register with Delay
	OCR1AL=(Config.delay_amount&0x0F);
	set_range();
	sbi(PORTD, SLEEP);	//Make sure accelerometer is not in sleep mode!
	
}

//Sends AT commands at 115200bps to configure SparkFun BGB203 Bluetooth Module
void config_bluetooth(void)
{
	//Enter Command Mode
	printf("$$$\n");
	delay_ms(50);

	//Disable the Config Timer
	printf("ST,0\n");
	delay_ms(50);

	//Activate Fast Data Mode
	printf("F,1\n");
	delay_ms(50);

}

static int uart_putchar(char c, FILE *stream)
{
  //if (c == '\n')
  // uart_putchar('\r', stream);
  
  loop_until_bit_is_set(UCSR0A, UDRE0);
  UDR0 = c;
  return 0;
}

uint8_t uart_getchar(void)
{
    while( !(UCSR0A & (1<<RXC0)) );
	return(UDR0);
}

static int witilt_putchar(char c)
{
  loop_until_bit_is_set(UCSR0A, UDRE0);
  UDR0 = c;
  return 0;
}

//General short delays
void delay_ms(uint16_t x)
{
  uint8_t y, z;
  for ( ; x > 0 ; x--){
    for ( y = 0 ; y < 40 ; y++){
      for ( z = 0 ; z < 40 ; z++){
        asm volatile ("nop");
      }
    }
  }
}

//Description: Writes an unsigned char(Data)  to the EEPROM at the given Address
//Pre: Unsigned Int Address contains address to be written to
//	 Unsigned Char Data contains data to be written
//Post: EEPROM "Address" contains the "Data"
//Usage: write_to_EEPROM(0, 'A');
void write_to_EEPROM(unsigned int Address, unsigned char Data)
{
    //Interrupts are globally disabled!
	
	while(EECR & (1<<EEPE)); //Wait for last Write to complete
	//May need to wait for Flash to complete also!
	EEAR = Address;			//Assign the Address Register with "Address"
	EEDR=Data;				//Put "Data" in the Data Register
	EECR |= (1<<EEMPE); 	//Write to Master Write Enable
	EECR |= (1<<EEPE);  	//Start Write by setting EE Write Enable
	
}

//Description: Reads the EEPROM data at "Address" and returns the character
//Pre: Unsigned Int Address is the address to be read
//Post: Character at "Address" is returned
//Usage: 	unsigned char Data;
//		Data=read_from_EEPROM(0);
unsigned char read_from_EEPROM(unsigned int Address)
{
	//Interrupts are globally disabled!
	
	while(EECR & (1<<EEPE));	//Wait for last Write to complete
	EEAR = Address;				//Assign the Address Register with "Address"
	EECR |= (1<<EERE); 			//Start Read by writing to EER
	return EEDR;				//EEPROM Data is returned
}

//Description: Returns the raw 16 bit accelerometer value for the given axis
//Pre: "axis" a character indicating the axis to be read ('x', 'y', or 'z')
//Post: Value of Axis is returned
//Usage: Sensor.x_val = get_TriAx_Val('x');
uint16_t get_TriAx_Val(uint8_t axis)
{
	//AVERAGE defines the number of samples to take from the ADC channel
	#define AVERAGE 16
	uint16_t temp, result=0;
	
	if(axis == 'x') ADMUX = 2;				//Select Channel 0 for X Axis
	else if(axis == 'y') ADMUX = 1;		//Select Channel 1 for Y Axis
	else if(axis == 'z') ADMUX = 0;		//Select Channel 2 for Z Axis
	else if(axis == 'b') ADMUX = 7;		//Select channel 7 for battery voltage
	else return 1025;
	
	for(uint8_t i = 0; i < AVERAGE; i++)		//Read ADC Port
	{
		ADCSRA = (1<<ADEN)|(1<<ADSC)|(1<<ADPS2)|(1<<ADPS1);
		
		//Conversion starts - wait for ADSC to clear
		while( (ADCSRA & (1<<ADSC)) != 0 );
		
		temp = ADCL;
		temp |= ADCH<<8;
		
		result += temp;		
	}
	
	//Check to see if the acceleration value exceeds the threshold!
	if(calibration_flag==0)
	{
		if((axis == 'x')&&((result/AVERAGE >= Calibration.X_Midpoint + Config.threshold)||(result/AVERAGE < Calibration.X_Midpoint - Config.threshold)))return result/AVERAGE;
		else if((axis == 'y')&&((result/AVERAGE >= Calibration.Y_Midpoint + Config.threshold)||(result/AVERAGE < Calibration.Y_Midpoint - Config.threshold)))return result/AVERAGE;
		else if((axis == 'z')&&((result/AVERAGE >= Calibration.Z_Midpoint + Config.threshold)||(result/AVERAGE < Calibration.Z_Midpoint - Config.threshold)))return result/AVERAGE;
		else if((axis == 'b')&&((uint16_t)result/AVERAGE <= (((uint32_t)Config.bthreshold*1024)/6600)))return result/AVERAGE;
		else return 1025;
	}
	else
		return result/AVERAGE;
}


void calibrate(void)
{
	uint16_t TempVal, tempmin, tempmax =0;
	uint8_t stop;

	calibration_flag=1;
	
	printf("\n\r\n\rX Midpoint: %d X G-Width: %d\n\r", Calibration.X_Midpoint, Calibration.X_1g);
	printf("Y Midpoint: %d Y G-Width: %d\n\r", Calibration.Y_Midpoint, Calibration.Y_1g);
	printf("Z Midpoint: %d Z G-Width: %d\n\r", Calibration.Z_Midpoint, Calibration.Z_1g);
	printf("Gyro Zero Value: %d\n\r", Gyro.zero);
	printf("To continue calibrating press 'space,' to exit, press 'x'\n\r");
	
	stop=uart_getchar();
	if(stop==32){
		for(uint8_t i='x'; i<='z'; i++)
		{
			if(i=='x')printf_P(rotatex_str);
			else if(i=='y')printf_P(rotatey_str);
			else if(i=='z')printf_P(rotatez_str);
			while(1)
			{
				TempVal = get_TriAx_Val(i);
				printf(" %d  \r", TempVal);
				if((UCSR0A & 0x80) == 0x80)
					{
						stop = UDR0;
						break;
					}
				delay_ms(100);
			}
			tempmax = TempVal;
			printf_P(calibrate_min_str);	
			while(1)
			{
				TempVal = get_TriAx_Val(i);
				printf(" %d  \r", TempVal);
				if((UCSR0A & 0x80) == 0x80)
				{
					stop = UDR0;
					break;
				}
				delay_ms(100);
			}
			tempmin = TempVal;
		
			if(i=='x')
			{
				Calibration.X_Midpoint = ((tempmax-tempmin)/2)+tempmin;		//Find the Midpoint
				Calibration.X_1g = tempmax - Calibration.X_Midpoint;
			}
			if(i=='y')
			{
				Calibration.Y_Midpoint = ((tempmax-tempmin)/2)+tempmin;		//Find the Midpoint
				Calibration.Y_1g = tempmax - Calibration.Y_Midpoint; 
			}
			if(i=='z')
			{
				Calibration.Z_Midpoint = ((tempmax-tempmin)/2)+tempmin;		//Find the Midpoint
				Calibration.Z_1g = tempmax - Calibration.Z_Midpoint; 
			}
		}
		printf_P(rotater_str);
		while(1)
			{
				Gyro.zero = adc_Read(RATE_CHANNEL);
				printf(" %d \r", Gyro.rate);
				if((UCSR0A & 0x80) == 0x80)
				{
					stop = UDR0;
					break;
				}
				delay_ms(100);
			}
			Gyro.zero = Gyro.rate;
	
		//Write X-Axis Calibration Values to EEPROM
		write_to_EEPROM(CalStartAdd + x_midpoint_addr + LOW_BYTE, (Calibration.X_Midpoint & 0xFF));
		write_to_EEPROM(CalStartAdd + x_midpoint_addr + HIGH_BYTE, ((Calibration.X_Midpoint>>8)&0xFF));
		write_to_EEPROM(CalStartAdd + x_1g_addr + LOW_BYTE, (Calibration.X_1g & 0xFF));
		write_to_EEPROM(CalStartAdd + x_1g_addr + HIGH_BYTE, ((Calibration.X_1g>>8)&0xFF));
	
		//Write Y-Axis Calibration Values to EEPROM
		write_to_EEPROM(CalStartAdd + y_midpoint_addr + LOW_BYTE, (Calibration.Y_Midpoint & 0xFF));
		write_to_EEPROM(CalStartAdd + y_midpoint_addr + HIGH_BYTE, ((Calibration.Y_Midpoint>>8)&0xFF));
		write_to_EEPROM(CalStartAdd + y_1g_addr + LOW_BYTE, (Calibration.Y_1g & 0xFF));
		write_to_EEPROM(CalStartAdd + y_1g_addr + HIGH_BYTE, ((Calibration.Y_1g>>8)&0xFF));	
	
		//Write Z-Axis Calibration Values to EEPROM
		write_to_EEPROM(CalStartAdd + z_midpoint_addr + LOW_BYTE, (Calibration.Z_Midpoint & 0xFF));
		write_to_EEPROM(CalStartAdd + z_midpoint_addr + HIGH_BYTE, ((Calibration.Z_Midpoint>>8)&0xFF));
		write_to_EEPROM(CalStartAdd + z_1g_addr + LOW_BYTE, (Calibration.Z_1g & 0xFF));
		write_to_EEPROM(CalStartAdd + z_1g_addr + HIGH_BYTE, ((Calibration.Z_1g>>8)&0xFF));
	
		//Write Gyroscope Calibration Values
		write_to_EEPROM(GyroStartAdd + zero_addr + LOW_BYTE, (Gyro.zero&0xFF));
		write_to_EEPROM(GyroStartAdd + zero_addr + HIGH_BYTE, ((Gyro.zero>>8)&0xFF));
	}
	
	calibration_flag=0;

}

void activate_channels(void)
{
	uint8_t channel_active=0;
	printf_P(activateX);
	channel_active = uart_getchar();
	if((channel_active=='y') | (channel_active=='Y'))TriAx.x_status = ACTIVE;
	else if((channel_active=='n') | (channel_active=='N'))TriAx.x_status=INACTIVE;
	
	printf_P(activateY);
	channel_active = uart_getchar();
	if((channel_active=='y') | (channel_active=='Y'))TriAx.y_status = ACTIVE;
	else if((channel_active=='n') | (channel_active=='N'))TriAx.y_status=INACTIVE;
	
	printf_P(activateZ);
	channel_active = uart_getchar();
	if((channel_active=='y') | (channel_active=='Y'))TriAx.z_status = ACTIVE;
	else if((channel_active=='n') | (channel_active=='N'))TriAx.z_status=INACTIVE;
	
	printf_P(activateB);
	channel_active = uart_getchar();
	if((channel_active=='y') | (channel_active=='Y'))TriAx.b_status = ACTIVE;
	else if((channel_active=='n') | (channel_active=='N'))TriAx.b_status=INACTIVE;
		
	printf_P(activateR);					
	channel_active = uart_getchar();
	if((channel_active=='y') | (channel_active=='Y'))Gyro.status = ACTIVE;
	else if((channel_active=='n') | (channel_active=='N'))Gyro.status=INACTIVE;
	
	write_to_EEPROM(TriAxStartAdd+x_status_addr, TriAx.x_status);
	write_to_EEPROM(TriAxStartAdd+y_status_addr, TriAx.y_status);
	write_to_EEPROM(TriAxStartAdd+z_status_addr, TriAx.z_status);
	write_to_EEPROM(TriAxStartAdd+b_status_addr, TriAx.b_status);
	write_to_EEPROM(GyroStartAdd+status_addr, Gyro.status);
}


void change_range_mode(void)
{
	uint8_t selection=0;
	
	printf_P(range_mode_str);

	selection = uart_getchar();
	Config.range_mode = selection;
	write_to_EEPROM(ConfigStartAdd+range_mode_addr, Config.range_mode);
	
	set_range();
}

void set_range(void)
{
	if(Config.range_mode==RANGE_15G)
	{
		cbi(PORTD, GS1);	//Range Mode = 1.5G
	}
	if(Config.range_mode==RANGE_6G)
	{
		sbi(PORTD, GS1);	//Range Mode = 6G
	}
}

void change_display_mode(void)
{
	uint8_t tdisplay_mode=0;
	printf_P(change_disp_str);
	tdisplay_mode = uart_getchar();
	while(1)
	{
		if((tdisplay_mode==GRAVITY)||(tdisplay_mode==RAW)||(tdisplay_mode==BINARY)||tdisplay_mode==DEGREE)
		{
			Config.display_mode=tdisplay_mode;
			write_to_EEPROM(ConfigStartAdd+display_mode_addr, Config.display_mode);
			break;
		}
		else if(tdisplay_mode=='x')break;
		else tdisplay_mode=uart_getchar();
	}
	if(Config.display_mode==DEGREE){	//All channels must be active for degree mode to work!
		TriAx.x_status=ACTIVE;
		TriAx.y_status=ACTIVE;
		TriAx.z_status=ACTIVE;
	}
	
}

void set_threshold(void)
{
	uint8_t temp_thresh, change=0;
	uint16_t bthresh=0;
	temp_thresh=Config.gthreshold;
	
	printf("\n\rChange Tilt Threshold:\n\r");
	printf_P(incrdecr);
	while(1)
	{
		printf("Gravity Value: %d.%d  \r", temp_thresh/10, temp_thresh%10);
		change = uart_getchar();
		
		if((change == 'i') | (change == 'I'))temp_thresh++;
		else if((change == 'd') | (change == 'D'))
		{
			if(temp_thresh > 0)	temp_thresh--;
		}
		else if((change == 'x') | (change == 'X'))break;
	}
	Config.gthreshold = temp_thresh;
	
	printf("\n\rChange Gyro Threshold:\n\r");
	printf_P(incrdecr);
	
	temp_thresh=Config.rthreshold;
	while(1)
	{
		printf("Rotational Value: %d  \r", temp_thresh);
		change=uart_getchar();
		if((change == 'i') | (change == 'I'))temp_thresh++;
		else if((change=='d') | (change == 'D'))
		{
			if(temp_thresh>0)temp_thresh--;
		}
		else if((change=='x') | (change == 'X'))break;
	}
	Config.rthreshold=temp_thresh;
	
	
	printf("\n\rChange Battery Threshold\n\r");
	printf_P(incrdecr);
	bthresh=Config.bthreshold;
	while(1)
	{
		printf("Millivolts: %d  \r", bthresh);
		change=uart_getchar();
		if((change == 'i') | (change == 'I'))bthresh+=100;
		else if((change=='d') | (change == 'D'))
		{
			if(bthresh>0)bthresh-=100;
		}
		else if((change=='x') | (change == 'X'))break;
	}
	Config.bthreshold = bthresh;
		
	
	write_to_EEPROM(ConfigStartAdd+rthreshold_addr, Config.rthreshold);
	write_to_EEPROM(ConfigStartAdd + gthreshold_addr, Config.gthreshold);
	write_to_EEPROM(ConfigStartAdd + bthreshold_addr + LOW_BYTE, (Config.bthreshold&0XFF));
	write_to_EEPROM(ConfigStartAdd + bthreshold_addr + HIGH_BYTE, ((Config.bthreshold>>8)&0xFF));
	
	if(Config.range_mode==RANGE_15G)Config.threshold = Config.gthreshold*25;
	else if(Config.range_mode==RANGE_6G)Config.threshold = Config.gthreshold*6;
	write_to_EEPROM(ConfigStartAdd+threshold_addr, Config.threshold);
	
}


void set_output_freq(void)
{
	uint32_t display_freq=0;
	uint8_t choice, mod;
	
	display_freq=divided_clock/Config.delay_amount;
	mod=display_freq%10;
	if(mod!=0)display_freq-=mod;
	printf("\n\rChange Display Frequency:\n\r [i]ncrease\n\r [d]ecrease\n\r [x] Exit\n\r");
	
	while(1)
	{
		if(Config.display_mode == DEGREE && display_freq > 50) display_freq = 50;
		if(Config.display_mode == GRAVITY && display_freq > 135) display_freq = 135;
		if(Config.display_mode == RAW && display_freq > 220) display_freq = 220;
		if(display_freq > 350) display_freq=350;
		printf(": = %ldHz  \r ", display_freq); //Print current delay
		choice=uart_getchar();
		if((choice == 'i') | (choice == 'I'))display_freq+=10;
		if(((choice == 'd') | (choice == 'D')) && display_freq>=10)display_freq-=10;
		if(choice == 'x')break;
	}
	
	Config.delay_amount=divided_clock/display_freq;
	write_to_EEPROM(ConfigStartAdd + delay_amount_addr + LOW_BYTE, (Config.delay_amount & 0xFF));
	write_to_EEPROM(ConfigStartAdd + delay_amount_addr + HIGH_BYTE, ((Config.delay_amount>>8)&0xFF));
	
	OCR1AH=(Config.delay_amount>>8);		//Load Compare Register with Delay
	OCR1AL=(Config.delay_amount&0x0F);
}

void rawDisplay(void)
{
	uint32_t Gvalue=0;

	while( !(UCSR0A & (1<<RXC0)) )	//Do Loop until key is pressed
	{
		while(values_updated != 1);		//Poll Updated Flag
		values_updated = 0;				//Clear updated flag
		cbi(PORTD, STATUS_LED);
		
		if((TriAx.x_status == 1) && (TriAx.x_val != 1025))		//Print X Values
		{
			printf("X=");		
			if(Config.display_mode== RAW)
			{
				printf("%d", TriAx.x_val);
			}
			else
			{
				if(TriAx.x_val>Calibration.X_Midpoint)
				{
					Gvalue=TriAx.x_val-Calibration.X_Midpoint;
				}
				else
				{
					printf("-");
					Gvalue=Calibration.X_Midpoint-TriAx.x_val;
				}	
				print_decimal((Gvalue*1000)/Calibration.X_1g, 3, 4);
			}
		}
		
		if((TriAx.y_status == 1) && (TriAx.y_val != 1025))		//Print Y Values
		{
			printf("Y=");		
			if(Config.display_mode== RAW)
			{
				printf("%d", TriAx.y_val);
			}
			else
			{
				if(TriAx.y_val>Calibration.Y_Midpoint)
				{
					Gvalue=TriAx.y_val-Calibration.Y_Midpoint;
				}
				else
				{
					printf("-");
					Gvalue=Calibration.Y_Midpoint-TriAx.y_val;
				}	
				print_decimal((Gvalue*1000)/Calibration.Y_1g, 3, 4);
			}
		}
		
		if((TriAx.z_status == 1) && (TriAx.z_val != 1025))		//Print Z Values
		{
			printf("Z=");		
			if(Config.display_mode== RAW)
			{
				printf("%d", TriAx.z_val);
			}
			else	//Display_mode == GRAVITY
			{
				if(TriAx.z_val>Calibration.Z_Midpoint)
				{
					Gvalue=TriAx.z_val-Calibration.Z_Midpoint;
				}
				else
				{
					printf("-");
					Gvalue=Calibration.Z_Midpoint-TriAx.z_val;
				}	
				print_decimal((Gvalue*1000)/Calibration.Z_1g, 3, 4);
			}
		}
		
		if((TriAx.b_status == 1) && (TriAx.b_val != 1025))		//Print Z Values
		{
			printf("B=");		
			if(Config.display_mode== RAW)
			{
				printf("%d", TriAx.b_val);
			}
			else	//Display_mode == GRAVITY
			{
				Gvalue=((3300*(uint32_t)TriAx.b_val)/1024)*2;
				print_decimal(Gvalue, 3, 4);
			}
		}
		
		if((Gyro.status == 1) && (Gyro.rate != 1025))
		{
			printf("R=");
			if(Config.display_mode==RAW)
			{
				printf("%d", Gyro.rate);
			}
			else
			{
				if(Gyro.rate>Gyro.zero)Gvalue=Gyro.rate-Gyro.zero;
				else
				{
					printf("-");
					Gvalue=Gyro.zero-Gyro.rate;
				}
				print_decimal((Gvalue*100)/gyro_scale,0,3);
			}
		}
		
		putchar('\n');
		putchar('\r');
		
		sbi(PORTD, STATUS_LED);
	}
	Gvalue=UDR0;
}
void binaryDisplay(void)
{
	uint8_t go=0, input=0, shighbyte=0;
	uint16_t sample_number=0, xhigh=0, yhigh=0, zhigh=0, rhigh=0, bhigh=0;

	putchar('#'); //Start with 0x01
    putchar('R'); //0x20
    putchar('$'); //End with 0x03
	while(1)
	{
		while(values_updated != 1);		//Poll Updated Flag
		values_updated = 0;				//Clear updated flag
		cbi(PORTD, STATUS_LED);
		if(go==1)
		{
			sample_number++;
			if((sample_number&0xFF)==10)sample_number++;
		    putchar('#'); //Start with 0x01
            putchar('@'); //Reading output starts with 0x22
			shighbyte=(sample_number>>8);
			putchar(shighbyte); //
			putchar(sample_number&0xFF); //Display Sample Number
			if (TriAx.x_status == ACTIVE)
            {
				xhigh= (TriAx.x_val>>8)&0xFF;
                putchar((uint8_t)xhigh); 
                putchar((uint8_t)TriAx.x_val&0xFF);  
            }
              
            if (TriAx.y_status == ACTIVE)
            {
				yhigh= (TriAx.y_val>>8)&0xFF;
                putchar((uint8_t)yhigh); 
                putchar((uint8_t)TriAx.y_val&0xFF); 
            }
               
            if (TriAx.z_status == ACTIVE)
            {
				zhigh= (TriAx.z_val>>8)&0xFF;
                putchar((uint8_t)zhigh); 
                putchar((uint8_t)TriAx.z_val&0xFF); 
            }
            
			if (TriAx.b_status == ACTIVE)
            {
				bhigh= (TriAx.b_val>>8)&0xFF;
                putchar((uint8_t)bhigh); 
                putchar((uint8_t)TriAx.b_val&0xFF); 
            }
			
			if(Gyro.status== ACTIVE)
			{
				rhigh= (Gyro.rate>>8)&0xFF;
				putchar((uint8_t)rhigh);
				putchar((uint8_t)Gyro.rate&0xFF);
			}
            putchar('$'); //End with 0x03
		}
		//Read incoming UART commands
        if((UCSR0A & 0x80) == 0x80)
        {        
            input = UDR0;

            if ((input == 32) && (go == 0)) //Config mode - fires when XOff or spacebar is received from terminal
            {
                break;
            }
    
            else if (input == 'R') //0x10 Ready?
            {
                //Respond with 0x20
                putchar('#'); //Start with 0x01
                putchar('R'); //0x20
                putchar('$'); //End with 0x03
            }
    
            else if (input == 'S') //0x12 Start Sampling
            {
                //Respond with 0x20 and start sampling
                putchar('#'); //Start with 0x01
                putchar('R'); //0x20
                putchar('$'); //End with 0x03
                go = 1;
            }
    
            else if (input == 'A') //0x13 Stop Sampling
            {
                //Respond with 0x20 and stop sampling
                putchar('#'); //Start with 0x01
                putchar('R'); //0x20
                putchar('$'); //End with 0x03
                go = 0;
            }
    
        }
		sbi(PORTD, STATUS_LED);
	}
}


//Takes an int and prints a decimal in a given place
//(87004, 3, 0) = 87.004
//10 digit max = 1234567890
//pad will put zeros in front to make the length constant
//(87004, 3, 7) = 0087.004
void print_decimal(uint32_t number, uint8_t place, uint8_t pad)
{
    uint8_t i;
    uint8_t nums[10];
    
    for(i = 0 ; i < 11 ; i++) nums[i] = 0;
    
    for(i = 10 ; number > 0 ; i--)
    {
        nums[i] = number % 10;
        number /= 10;
    }
    
    //Pad the number of spots
    if (pad <= 10 && (pad > (10 - i)) ) i = 10 - pad;
    
    for(i++ ; i < 11 ; i++)
    {
        printf("%d", nums[i]);
        
        //Check for decimal printing
        if ( ((i + place) == 10) && (place != 0) ) printf(".");
    }
}

uint8_t spi_exch(uint8_t output)
{
	SPDR = output;				//Send Output to the SPI Data Register to initialize "Send"
	while(!(SPSR & 1<<SPIF));	//Wait for SPI to finish sending
	return SPDR;				//Return the received byte
}

uint16_t spi_full(uint8_t cmd, uint16_t sdat)
{
	union uint16_t{
		uint16_t ADCvalue;
		uint8_t byte[2];
	}inp;
	
	inp.ADCvalue=sdat;
	cbi(PORTB, PIN2);
	spi_exch(cmd);
	if(!(cmd & 0x80))delay_ms(6);
	inp.byte[1]=spi_exch(inp.byte[1]);
	inp.byte[0]=spi_exch(inp.byte[0]);
	sbi(PORTB, PIN2);
	return inp.ADCvalue;
}

uint16_t adcc(uint8_t value)
{
	return spi_full(0x90+value, 0);
}

uint16_t adcr(void)
{
	return spi_full(0x80, 0);
}
uint16_t adc_Read(uint8_t value)
{
	uint16_t ans=0;
	
	if(value==RATE_CHANNEL)adcc(RATE_CHANNEL);
	else adcc(TEMP_CHANNEL);
	delay_ms(1);
	ans=adcr();
	Gyro.rate=ans>>1;
	Gyro.rate &= ADC_MASK;
	
	if(calibration_flag==0)
	{
		if((Gyro.rate*100 >= Gyro.zero*100 + Config.rthreshold*341)||(Gyro.rate*100 < Gyro.zero*100 - Config.rthreshold*341))return ans;
		else Gyro.rate=1025;
	}
		
	return ans;
}




void degreeDisplay(void)
{
	uint32_t Gvalue=0;
	double Degree=0;
	double xvalue, yvalue, zvalue =0;
	uint8_t xquad=0, yquad=0, zquad=0;

	while( !(UCSR0A & (1<<RXC0)) )	//Do Loop until key is pressed
	{
		while(values_updated != 1);		//Poll Updated Flag
		values_updated = 0;				//Clear updated flag
		cbi(PORTD, STATUS_LED);
		if(TriAx.x_val>Calibration.X_Midpoint){
			Gvalue=TriAx.x_val-Calibration.X_Midpoint;
			xquad = POSITIVE;
		}
		else{
			Gvalue=Calibration.X_Midpoint-TriAx.x_val;
			xquad = NEGATIVE;
		}
		xvalue=(double)Gvalue/Calibration.X_1g;
		
		if(TriAx.y_val>Calibration.Y_Midpoint){
			Gvalue=TriAx.y_val-Calibration.Y_Midpoint;
			yquad = POSITIVE;
		}
		else{
			Gvalue=Calibration.Y_Midpoint-TriAx.y_val;
			yquad = NEGATIVE;
		}
		yvalue=(double)Gvalue/Calibration.Y_1g;
	
		if(TriAx.z_val>Calibration.Z_Midpoint){
			Gvalue=TriAx.z_val-Calibration.Z_Midpoint;
			zquad = POSITIVE;
		}
		else{
			Gvalue=Calibration.Z_Midpoint-TriAx.z_val;
			zquad = NEGATIVE;
		}	
		zvalue=(double)Gvalue/Calibration.Z_1g;
	
		if((TriAx.x_status == 1) && (TriAx.x_val != 1025))		//Print X Values
		{		
			Degree = (180/3.14)*atan2(xvalue, sqrt((yvalue*yvalue)+(zvalue*zvalue)));
			if((xquad==POSITIVE)&&(zquad==POSITIVE));						//X is in 1st Quadrant
			if((xquad==POSITIVE)&&(zquad==NEGATIVE))Degree=180-Degree;	//x is in 2nd quadrant
			if((xquad==NEGATIVE)&&(zquad==NEGATIVE))Degree+=180;			//x is in 3rd quadrant
			if((xquad==NEGATIVE)&&(zquad==POSITIVE))Degree=360-Degree;	//x is in 4th quadrant
			printf("X=%05.0lf", Degree);
		}	
		
		if((TriAx.y_status == 1) && (TriAx.y_val != 1025))		//Print X Values
		{
			Degree = (180/3.14)*atan2(yvalue, sqrt((xvalue*xvalue)+(zvalue*zvalue)));
			if((yquad==POSITIVE)&&(zquad==POSITIVE));						//y is in 1st quadrant
			if((yquad==POSITIVE)&&(zquad==NEGATIVE))Degree=180-Degree;	//Y is in 2nd quadrant
			if((yquad==NEGATIVE)&&(zquad==NEGATIVE))Degree+=180;			//Y is in 3rd quadrant
			if((yquad==NEGATIVE)&&(zquad==POSITIVE))Degree=360-Degree;	//y is in 4th quadrant
			printf("Y=%05.0lf", Degree);	
		}
		
		
		/*if((TriAx.z_status == 1) && (TriAx.z_val != 1025))		//Print X Values
		{
			Degree=(180/3.14)*atan2(sqrt((xvalue*xvalue)+(yvalue*yvalue)),zvalue);
			if((xquad == POSITIVE)&&(zquad==POSITIVE))					//X is in 1st Quadrant
			if((xquad==POSITIVE)&&(zquad==NEGATIVE))Degree=180-Degree;//x is in 2nd quadrant
			if((xquad==NEGATIVE)&&(zquad==NEGATIVE))Degree+=180;		//x is in 3rd quadrant
			if((xquad==NEGATIVE)&&(zquad==POSITIVE))Degree=360-Degree;//x is in 4th quadrant
			printf("Z-Degree: %3.0lf", Degree);
		}*/
		
		
		if((Gyro.status == 1) && (Gyro.rate != 1025))
		{
			printf("R=");
			if(Gyro.rate>Gyro.zero)Gvalue=Gyro.rate-Gyro.zero;
			else
			{
				printf("-");
				Gvalue=Gyro.zero-Gyro.rate;
			}
			print_decimal((Gvalue*100)/gyro_scale,0,3);
		}
		
		if((TriAx.b_status == 1) && (TriAx.b_val != 1025))		//Print Z Values
		{
			printf("B=");		
			Gvalue=((3300*(uint32_t)TriAx.b_val)/1024)*2;
			print_decimal(Gvalue, 3, 4);
		}
		
		putchar('\n');
		putchar('\r');
		sbi(PORTD, STATUS_LED);
	}
	
}

